import 'package:flutter_code_push/constant/WTVMConstant.dart';
import 'package:flutter_code_push/memory/WTClassMemory.dart';
import 'package:flutter_code_push/memory/WTUnitMemory.dart';
import 'package:flutter_code_push/pointer/WTClassPointer.dart';
import 'package:flutter_code_push/sdkBridge/WTSDKBridge.dart';
import 'package:flutter_code_push/generate/WTVMBaseType.dart';

/// 万物皆对象、部分对象保留左值（父级），用作赋值对象用、对象无需感知从何而来。
/// 保存环境值
class Environment {
  static Function newInstance = () => Environment();
  
  Map<String, dynamic> store = {};
  Environment outer;

  Environment();

  int defaultCount;
  /// 获取当前所在的Env
  /// 返回类型 Environment、WTUnitMemory
  dynamic _obtainCurrentEnv(String attr, {bool isDirect = false}) {
    if(isDebug) {
      if(defaultCount == null)
        defaultCount = 0;

      defaultCount++;
      if(attr == "isDirectBreak" && defaultCount == 50)
        int x=10;
    }

    if(store.containsKey(attr))
      return this;

    if(isDirect == true)
      return null;

    if(isDirect == false) {
      if(store.containsKey(WTVMConstant.superKeyword)) {
        var superValue = store[WTVMConstant.superKeyword];
        WTClassPointer superPointer;
        Environment superEnv;
        if(superValue is WTClassPointer) {
          superPointer = superValue;
          superEnv = superPointer?.selfEnv;
        }
        else if(superPointer is Environment) {
          superEnv = superValue;
        }
        superEnv = superEnv?._obtainCurrentEnv(attr, isDirect: isDirect);
        if(superEnv != null)
          return superEnv;

        WTClassPointer thisPointer = store[WTVMConstant.thisKeyword];
        WTClassMemory staticMemory = thisPointer.staticMemory;
        bool staticContains = staticMemory.staticEnv.store.containsKey(attr);
        Environment staticEnv = staticContains ? staticMemory.staticEnv : null;
        if(staticEnv != null)
          return thisPointer.selfEnv;
      }

      if(store.containsKey(WTVMConstant.importUnitList)) {
        Map importMap = store[WTVMConstant.importUnitList];
        var keys = importMap.keys.toList();
        int size = keys.length;
        for (var i = 0; i < size; ++i) {
          var key = keys[i];
          var value = importMap[key];
          if(value is Environment) {
            Environment value = importMap[key];
            Environment out = value._obtainCurrentEnv(attr, isDirect: true);
            if(out != null) {
              return out;
            }
          }else if(value is WTUnitMemory) {
            WTUnitMemory unitMemory = value;
            if(unitMemory.containsKey(attr)) {
              return unitMemory;
            }
          }
        }
      }
    }

    return outer?._obtainCurrentEnv(attr, isDirect: isDirect);
  }

  dynamic get(String attr, {bool isDirect = false}) {
    if(attr == "_selectedIndex")
      int x=10;

    if(attr == null)
      throw "The obtain attribute value is null";

    var tempValue = _obtainCurrentEnv(attr, isDirect: isDirect);
    if(tempValue is Environment) {
      var curEnv = tempValue;
      var value = curEnv != null ? curEnv.store[attr] : null;
      if(value == null && isDirect == false) {
        if(curEnv != null && curEnv.store.containsKey(WTVMConstant.thisKeyword)) {
          WTClassPointer pointer = curEnv.get(WTVMConstant.thisKeyword);
          var isGetOrSetMethod = pointer.declaration.isGetOrSetMethod(attr);
          if(isGetOrSetMethod) {
            return pointer.getValue(attr);
          }
          else if(pointer.staticMemory.containsKey(attr)) {
            return pointer.getValue(attr);
          }
        }
      }

      /// 检测是否是原生的get方法
      if(value is WTVMBaseType) {
        if(value.getAttributeMap?.containsKey(attr) == true) {
          var f = value.getAttributeMap[attr];
          return f();
        }
      }
      return value;
    }
    else if(tempValue is WTUnitMemory) {
      WTUnitMemory unitMemory = tempValue;
      return unitMemory.getValue(attr);
    }
  }

  /// isDirect 是否直接存储
  void set(String attr, dynamic object, {bool isDirect = false, bool isOverride = true}) {
    if(attr == "imageUrl" && object is double) {
      int x=1;
    }

    if(attr == null || attr == "")
      throw "变量名不能为null";

    if(outer == null || isDirect) {
      if(isOverride == true || store.containsKey(attr) == false)
        store[attr] = object;
    }
    else {
      var tempValue = _obtainCurrentEnv(attr);
      if(tempValue  == null)
        throw "赋值找不到环境 $attr $Object";

      if(tempValue is Environment) {
        var curEnv = tempValue;

        if(curEnv.store.containsKey(WTVMConstant.thisKeyword)) {
          WTClassPointer pointer = curEnv.get(WTVMConstant.thisKeyword);
          var isGetOrSetMethod = pointer.declaration.isGetOrSetMethod(attr);
          if(isGetOrSetMethod) {
            pointer.setValue(attr, object);
            return;
          }
        }

        /// 检测调用原生类的，set函数
        var targetValue = curEnv.store[attr];
        if(targetValue is WTVMBaseType) {
          WTVMBaseType b = targetValue;
          Map setMap = b.setAttributeMap;
          if(setMap?.containsKey(attr)) {
            var f = setMap[attr];
            return f(object);
          }
        }
        else {
          curEnv.store[attr] = object;
        }
      }
      else if(tempValue is WTUnitMemory) {
        WTUnitMemory unitMemory = tempValue;
        unitMemory.setValue(attr, object);
      }
    }
  }

  void del(String attr) {
    store.remove(attr);
  }

  bool containsKey(String attr) {
    return store.containsKey(attr);
  }
}