import 'package:flutter_code_push/Environment.dart';
import 'package:flutter_code_push/constant/WTVMConstant.dart';
import 'package:flutter_code_push/declaration/WTClassDeclaration.dart';
import 'package:flutter_code_push/declaration/WTConstructorDeclaration.dart';
import 'package:flutter_code_push/declaration/WTMethodDeclaration.dart';
import 'package:flutter_code_push/declaration/WTMethodInvocation.dart';
import 'package:flutter_code_push/declaration/WTSimpleIdentifierImpl.dart';
import 'package:flutter_code_push/declaration/WTTypeArgumentList.dart';
import 'package:flutter_code_push/declaration/WTTypeName.dart';
import 'package:flutter_code_push/generate/WTVMBaseType.dart';
import 'package:flutter_code_push/memory/WTClassMemory.dart';
import 'package:flutter_code_push/sdkBridge/WTSDKBridge.dart';

/// 类指针
class WTClassPointer {

  WTClassDeclaration declaration;
  WTClassMemory superClassMemory;

  Environment selfEnv;

  WTClassPointer superPointer;

  /// 继承的原生类
  WTVMBaseType superType;

  /// 类静态对象
  WTClassMemory staticMemory;

  void initializer(WTClassDeclaration declaration,
      WTClassMemory staticMemory, Environment env) {
    this.declaration = declaration;
    this.staticMemory = staticMemory;

    selfEnv = Environment.newInstance();
    selfEnv.set(WTVMConstant.thisKeyword, this);

    var superClass = declaration.extendsClause?.superClass;
    if(superClass != null) {
      if(declaration.superDeclaration != null) {
        WTClassMemory classMemory = env.get(superClass.typeName);
        superClassMemory = classMemory;
      }
      else {
        superType = env.get(superClass.typeName) as WTVMBaseType;
      }

      if(superClassMemory == null && superType == null) {
        debugPrint("未知的继承 $superClass");
      }
    }

    var implementsClause = declaration.implementsClause;
    var interfaces = implementsClause?.interfaces;
    superClass = (interfaces?.length ?? 0) >= 1 ? interfaces[0] : null;
    if(superClass != null) {
      if(declaration.superDeclaration != null) {
        WTClassMemory classMemory = env.get(superClass.typeName);
        superClassMemory = classMemory;
      }
      else {
        superType = env.get(superClass.typeName) as WTVMBaseType;
      }

      if(superClassMemory == null && superType == null) {
        debugPrint("未知的继承 $superClass");
      }
    }

    selfEnv.outer = env;
    selfEnv.set(WTVMConstant.superKeyword, superPointer, isDirect:true);
    WTClassMemory.registerStaticEnv(selfEnv, declaration, false);
  }

  /// 执行构造函数
  WTClassPointer executeConstructor(WTConstructorDeclaration constructor,
      List positionalArguments,
      Map<Symbol, dynamic> namedArguments)  {
    if(constructor == null)
      constructor = declaration.constructor;
    return constructor?.execute(selfEnv, positionalArguments: positionalArguments, namedArguments: namedArguments);
  }


  /// 调用get函数
  dynamic getValue(String attrName) {
    if(attrName == 'tabSearch')
      int x=10;

    if(staticMemory.containsKey(attrName)) {
      return staticMemory.getValue(attrName);
    }
    else if(declaration.isGetOrSetMethod(attrName, true)) {
      if(declaration.isGetOrSetMethod(attrName)) {
        WTMethodDeclaration m = declaration.getClassMethod(attrName, MethodPropertyKeyword.get);
        if(m != null) {
          return m.execute(selfEnv);
        }else {
          return selfEnv.get(attrName);
        }
      }
      else {
        return superPointer.getValue(attrName);
      }
    }
    else {
      return selfEnv.get(attrName);
    }
  }

  void setValue(String attrName, dynamic value) {
    if(staticMemory.containsKey(attrName)) {
      return staticMemory.setValue(attrName, value);
    }
    else if(declaration.isGetOrSetMethod(attrName, true)) {
      if(declaration.isGetOrSetMethod(attrName)) {
        WTMethodDeclaration m = declaration.getClassMethod(attrName, MethodPropertyKeyword.set);
        return WTMethodInvocation.executeMethod(selfEnv, m, [value]);
      }else {
        return superPointer.setValue(attrName, value);
      }
    }
    else {
      return selfEnv.set(attrName, value);
    }
  }

  dynamic executeMethod(dynamic methodName, List positionalArguments, [Map<Symbol, dynamic> namedArguments]) {
    if(methodName == 'setSongs')
      int x=10;

    dynamic method = methodName;
    if(methodName is String) {
      method = getExecuteMethod(methodName);
    }
    if(method == null) {
      debugPrint("execute $methodName is null");
      return null;
    }

    var outValue = WTMethodInvocation.executeMethod(selfEnv, method, positionalArguments, namedArguments);
    if(method is WTMethodDeclaration) {
      var returnType = method.returnType;
      if(returnType is WTTypeName) {
        var nameDeclaration = returnType.nameDeclaration;
        WTTypeArgumentList typeArguments = returnType.typeArguments;
        if(nameDeclaration is WTSimpleIdentifierImpl) {
          if(nameDeclaration.identifierName == 'Future' && typeArguments != null) {
            var arguments = typeArguments.arguments;
            int size = arguments.length;
            for(var i = 0; i < size; ++i) {
              WTTypeName typeName = arguments[i];
              var nameDeclaration = typeName.nameDeclaration;
              if(nameDeclaration is WTSimpleIdentifierImpl) {
                var identifierName = nameDeclaration.identifierName;

                Future v = outValue as Future;
                switch(identifierName) {
                  case 'Map':
                    Future<Map> o = v.then((value) => value as Map);
                    return o;
                    break;

                  case 'List':
                    Future<List> o = v.then((value) => value as List);
                    return o;
                    break;
                }
              }
            }
          }
        }
      }
    }
    return outValue;
  }

  dynamic getExecuteMethod(String methodName) {
    WTMethodDeclaration m = declaration.getClassMethod(methodName);
    var f = ((m == null) ? selfEnv.get(methodName, isDirect: true) : null);
    return m ?? f;
  }

  @override
  dynamic noSuchMethod(Invocation invocation) {
    String methodName;
    if(invocation.memberName != null) {
      methodName = "${invocation.memberName}";
      methodName = methodName.substring(8, methodName.length-2);
    }

    dynamic method = getExecuteMethod(methodName);

    if(invocation.isGetter == true) {
      return method;
    }
    else if(invocation.isSetter == true) {
      throw "Not yet implemented！";
    }
    else if(invocation.isMethod == true) {
      return executeMethod(method, invocation.positionalArguments, invocation.namedArguments);
    }
  }

// Map<String, dynamic> toJson() {
  //   final Map<String, dynamic> data = new Map<String, dynamic>();
  //   return data;
  // }


}